==> jei-1.19.4-fabric-13.1.0.8-sources.jar: mezz/jei/gui/overlay/IngredientListRenderer.java <==
package mezz.jei.gui.overlay;

import com.mojang.blaze3d.systems.RenderSystem;
import mezz.jei.api.ingredients.IIngredientRenderer;
import mezz.jei.api.ingredients.IIngredientType;
import mezz.jei.api.ingredients.ITypedIngredient;
import mezz.jei.api.runtime.IEditModeConfig;
import mezz.jei.api.runtime.IIngredientManager;
import mezz.jei.common.util.ErrorUtil;
import mezz.jei.common.util.ImmutableRect2i;
import net.minecraft.class_332;
import net.minecraft.class_4587;
import mezz.jei.common.config.IClientToggleState;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Stream;

public class IngredientListRenderer {
	private static final int BLACKLIST_COLOR = 0xFFFF0000;

	private final List<IngredientListSlot> slots = new ArrayList<>();
	private final ElementRenderersByType renderers = new ElementRenderersByType();
	private final IEditModeConfig editModeConfig;
	private final IClientToggleState toggleState;
	private final IIngredientManager ingredientManager;

	private int blocked = 0;

	public IngredientListRenderer(IEditModeConfig editModeConfig, IClientToggleState toggleState, IIngredientManager ingredientManager) {
		this.editModeConfig = editModeConfig;
		this.toggleState = toggleState;
		this.ingredientManager = ingredientManager;
	}

	public void clear() {
		slots.clear();
		renderers.clear();
		blocked = 0;
	}

	public int size() {
		return slots.size() - blocked;
	}

	public void add(IngredientListSlot ingredientListSlot) {
		slots.add(ingredientListSlot);
	}

	public Stream<IngredientListSlot> getSlots() {
		return slots.stream()
			.filter(s -> !s.isBlocked());
	}

	public void set(final int startIndex, List<ITypedIngredient<?>> ingredientList) {
		renderers.clear();
		blocked = 0;

		int i = startIndex;
		for (IngredientListSlot ingredientListSlot : slots) {
			if (ingredientListSlot.isBlocked()) {
				ingredientListSlot.clear();
				blocked++;
			} else {
				if (i >= ingredientList.size()) {
					ingredientListSlot.clear();
				} else {
					ITypedIngredient<?> ingredient = ingredientList.get(i);
					set(ingredientListSlot, ingredient);
				}
				i++;
			}
		}
	}

	private <V> void set(IngredientListSlot ingredientListSlot, ITypedIngredient<V> value) {
		ElementRenderer<V> renderer = new ElementRenderer<>(value);
		ingredientListSlot.setIngredientRenderer(renderer);
		IIngredientType<V> ingredientType = value.getType();
		renderers.put(ingredientType, renderer);
	}

	public void render(class_4587 poseStack) {
		for (IIngredientType<?> ingredientType : renderers.getTypes()) {
			renderIngredientType(poseStack, ingredientType);
		}
	}

	private <T> void renderIngredientType(class_4587 poseStack, IIngredientType<T> ingredientType) {
		Collection<ElementRenderer<T>> slots = renderers.get(ingredientType);
		IIngredientRenderer<T> ingredientRenderer = ingredientManager.getIngredientRenderer(ingredientType);
		for (ElementRenderer<T> slot : slots) {
			renderIngredient(poseStack, slot, ingredientRenderer);
		}
	}

	private <T> void renderIngredient(class_4587 poseStack, ElementRenderer<T> slot, IIngredientRenderer<T> ingredientRenderer) {
		ITypedIngredient<T> typedIngredient = slot.getTypedIngredient();
		ImmutableRect2i area = slot.getArea();
		int slotPadding = slot.getPadding();
		if (toggleState.isEditModeEnabled()) {
			renderEditMode(poseStack, area, slotPadding, editModeConfig, typedIngredient);
			RenderSystem.enableBlend();
		}

		T ingredient = typedIngredient.getIngredient();
		try {
			int xPosition = area.getX() + slotPadding;
			int yPosition = area.getY() + slotPadding;
			poseStack.method_22903();
			{
				poseStack.method_46416(xPosition, yPosition, 0);
				ingredientRenderer.render(poseStack, ingredient);
			}
			poseStack.method_22909();
		} catch (RuntimeException | LinkageError e) {
			throw ErrorUtil.createRenderIngredientException(e, ingredient, ingredientManager);
		}
	}

	private static <T> void renderEditMode(class_4587 poseStack, ImmutableRect2i area, int padding, IEditModeConfig editModeConfig, ITypedIngredient<T> typedIngredient) {
		if (editModeConfig.isIngredientHiddenUsingConfigFile(typedIngredient)) {
			class_332.method_25294(poseStack, area.getX() + padding, area.getY() + padding, area.getX() + 16 + padding, area.getY() + 16 + padding, BLACKLIST_COLOR);
			RenderSystem.setShaderColor(1f, 1f, 1f, 1f);
		}
	}
}

==> jei-1.20-fabric-14.0.0.4-sources.jar: mezz/jei/gui/overlay/IngredientListRenderer.java <==
package mezz.jei.gui.overlay;

import com.mojang.blaze3d.systems.RenderSystem;
import mezz.jei.api.ingredients.IIngredientRenderer;
import mezz.jei.api.ingredients.IIngredientType;
import mezz.jei.api.ingredients.ITypedIngredient;
import mezz.jei.api.runtime.IEditModeConfig;
import mezz.jei.api.runtime.IIngredientManager;
import mezz.jei.common.util.ErrorUtil;
import mezz.jei.common.util.ImmutableRect2i;
import net.minecraft.class_332;
import net.minecraft.class_4587;
import mezz.jei.common.config.IClientToggleState;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Stream;

public class IngredientListRenderer {
	private static final int BLACKLIST_COLOR = 0xFFFF0000;

	private final List<IngredientListSlot> slots = new ArrayList<>();
	private final ElementRenderersByType renderers = new ElementRenderersByType();
	private final IEditModeConfig editModeConfig;
	private final IClientToggleState toggleState;
	private final IIngredientManager ingredientManager;

	private int blocked = 0;

	public IngredientListRenderer(IEditModeConfig editModeConfig, IClientToggleState toggleState, IIngredientManager ingredientManager) {
		this.editModeConfig = editModeConfig;
		this.toggleState = toggleState;
		this.ingredientManager = ingredientManager;
	}

	public void clear() {
		slots.clear();
		renderers.clear();
		blocked = 0;
	}

	public int size() {
		return slots.size() - blocked;
	}

	public void add(IngredientListSlot ingredientListSlot) {
		slots.add(ingredientListSlot);
	}

	public Stream<IngredientListSlot> getSlots() {
		return slots.stream()
			.filter(s -> !s.isBlocked());
	}

	public void set(final int startIndex, List<ITypedIngredient<?>> ingredientList) {
		renderers.clear();
		blocked = 0;

		int i = startIndex;
		for (IngredientListSlot ingredientListSlot : slots) {
			if (ingredientListSlot.isBlocked()) {
				ingredientListSlot.clear();
				blocked++;
			} else {
				if (i >= ingredientList.size()) {
					ingredientListSlot.clear();
				} else {
					ITypedIngredient<?> ingredient = ingredientList.get(i);
					set(ingredientListSlot, ingredient);
				}
				i++;
			}
		}
	}

	private <V> void set(IngredientListSlot ingredientListSlot, ITypedIngredient<V> value) {
		ElementRenderer<V> renderer = new ElementRenderer<>(value);
		ingredientListSlot.setIngredientRenderer(renderer);
		IIngredientType<V> ingredientType = value.getType();
		renderers.put(ingredientType, renderer);
	}

	public void render(class_332 guiGraphics) {
		for (IIngredientType<?> ingredientType : renderers.getTypes()) {
			renderIngredientType(guiGraphics, ingredientType);
		}
	}

	private <T> void renderIngredientType(class_332 guiGraphics, IIngredientType<T> ingredientType) {
		Collection<ElementRenderer<T>> slots = renderers.get(ingredientType);
		IIngredientRenderer<T> ingredientRenderer = ingredientManager.getIngredientRenderer(ingredientType);
		for (ElementRenderer<T> slot : slots) {
			renderIngredient(guiGraphics, slot, ingredientRenderer);
		}
	}

	private <T> void renderIngredient(class_332 guiGraphics, ElementRenderer<T> slot, IIngredientRenderer<T> ingredientRenderer) {
		ITypedIngredient<T> typedIngredient = slot.getTypedIngredient();
		ImmutableRect2i area = slot.getArea();
		int slotPadding = slot.getPadding();
		if (toggleState.isEditModeEnabled()) {
			renderEditMode(guiGraphics, area, slotPadding, editModeConfig, typedIngredient);
			RenderSystem.enableBlend();
		}

		T ingredient = typedIngredient.getIngredient();
		try {
			int xPosition = area.getX() + slotPadding;
			int yPosition = area.getY() + slotPadding;
			var poseStack = guiGraphics.method_51448();
			poseStack.method_22903();
			{
				poseStack.method_46416(xPosition, yPosition, 0);
				ingredientRenderer.render(guiGraphics, ingredient);
			}
			poseStack.method_22909();
		} catch (RuntimeException | LinkageError e) {
			throw ErrorUtil.createRenderIngredientException(e, ingredient, ingredientManager);
		}
	}

	private static <T> void renderEditMode(class_332 guiGraphics, ImmutableRect2i area, int padding, IEditModeConfig editModeConfig, ITypedIngredient<T> typedIngredient) {
		if (editModeConfig.isIngredientHiddenUsingConfigFile(typedIngredient)) {
			guiGraphics.method_25294(area.getX() + padding, area.getY() + padding, area.getX() + 16 + padding, area.getY() + 16 + padding, BLACKLIST_COLOR);
			RenderSystem.setShaderColor(1f, 1f, 1f, 1f);
		}
	}
}
